function [beta, zStar, pf,nFunctionEvals,flag] = ...
    HansoferLind2D(gFunction,relBetaError,maxFunctionEvals)
% 2D implementation of Hansofer-Lind Algorithm
if (nargin == 1)
    relBetaError = 0.01; % 1 percent error in beta accepted
    maxFunctionEvals = 1000;
end
if (nargin == 2)
    maxFunctionEvals = 1000;
end
betaOld = 0; % initialize reliability index
zBar = [0 0]; % initial guess
nFunctionEvals = 0; % keep track of funcion calls
flag = 1; % flag = 1 means we have converged.
while (1) % to avoid infinite loops
    [g,grad_g] = gFunction(zBar);
    nFunctionEvals = nFunctionEvals+3; % count 1 for g and 2 for gradient
    a1 = grad_g(1);
    a2 = grad_g(2);
    a0 = g - a1*zBar(1) - a2*zBar(2);
    betaNew = a0/sqrt(a1*a1+a2*a2); % update based on linear approximation
    zBar(1) = -a1*a0/(a1*a1+a2*a2);
    zBar(2) = -a2*a0/(a1*a1+a2*a2);
    if (abs(betaNew-betaOld)/betaNew < relBetaError) % we have converged 
        break;
    end
    if (nFunctionEvals > maxFunctionEvals) % to avoid infinite loop
        flag = 0; % we have not converged.
        break;
    end
    betaOld = betaNew;
end
beta = betaNew;
zStar = zBar;
pf = 1- Normal_Probability(0,1,beta);
