function [g,gradZ] = gTruss7BarDeflectionWithGradient(z,h)
if (nargin == 1)
    h = 1e-6; % step size for finite difference
end
g = gTruss7BarDeflection(z);
N = numel(z);
gradZ = zeros(1,N);
if (N ~= 7)
    disp('Size of z must be equal to 7');
    return;
end
if (nargout == 1)
    return; % no need to compute gradient
end
z0 = z; % save the value
for i = 1:N
    z = z0;
    z(i) = z(i) + h; % increment the i-th value of z
    gh = gTruss7BarDeflection(z);
    gradZ(i) = (gh-g)/h;
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [g] = gTruss7BarDeflection(z)
% Reliability function for a specific truss problem
t = truss7Bar(); % nominal model
ANominal = 1e-6;
AStdDev = 2.5e-8;
for i = 1:numel(z)
    A =  AStdDev*z(i) + ANominal;
    t = t.assignA(A,i);
end
t = t.assemble();
t = t.solve();
deltaNominal = 2.6233e-05; % pre-computed nominal deflection
deltaAllowable = 1.05*deltaNominal;
g = deltaAllowable -t.myDeformationMax;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function t = truss7Bar(Areas)
% Areas is a 7x1 vector
if (nargin == 0)
    Areas = 1e-6;
end
xy = [0.5 1.5 1.0 0 2.0; 1 1 0 0 0];% nodes
connectivity = [1 2; 1 3; 1 4; 2 3; 2 5; 3 4; 3 5]'; % connectivity
t = truss2d(xy,connectivity);
t = t.assignE(2e11);
t = t.assignA(Areas);
t = t.fixXofNodes([4 5]);
t = t.fixYofNodes([4 5]);
t = t.applyForce(1,[1;-2]);
t = t.applyForce(2,[2;0]);
end
